@@ -1,7 +1,7 @@
 """
 mkdocstrings package.
 
-Automatic documentation from source code, for mkdocs.
+Automatic documentation from source code, for MkDocs.
 
 This project was originally written for Python source code only, hence the name.
 It was then refactored to use external tools to load documentation. The main purpose
@@ -12,6 +12,6 @@ that mkdocstrings can now implement "handlers" for any given language, as long a
 a tool able to collect documentation in source files for that language.
 """
 
-from .plugin import MkdocstringsPlugin
+from mkdocstrings.plugin import MkdocstringsPlugin
 
 __all__ = ["MkdocstringsPlugin"]
