@@ -22,28 +22,35 @@ instruction:
       option_x: etc
 ```
 """
-
+import logging
 import re
-from typing import Tuple
-from xml.etree.ElementTree import XML, Element, ParseError  # nosec: we choose to trust the XML input
+from typing import Any, Tuple
+from xml.etree.ElementTree import XML, Element, ParseError  # noqa: S405 (we choose to trust the XML input)
 
 import yaml
+from jinja2.exceptions import TemplateNotFound
 from markdown import Markdown
+from markdown.blockparser import BlockParser
 from markdown.blockprocessors import BlockProcessor
 from markdown.extensions import Extension
 from markdown.util import AtomicString
-from mkdocs.utils import log
+from mkdocs.utils import warning_filter
 
-from .handlers import CollectionError, get_handler
+from mkdocstrings.handlers import CollectionError, get_handler
+
+log = logging.getLogger(__name__)
+log.addFilter(warning_filter)
 
 
-def atomic_brute_cast(tree: Element):
+def atomic_brute_cast(tree: Element) -> Element:
     """
     Cast every node's text into an atomic string to prevent further processing on it.
 
     Since we generate the final HTML with Jinja templates, we do not want other inline or tree processors
     to keep modifying the data, so this function is used to mark the complete tree as "do not touch".
 
+    Reference: issue [Python-Markdown/markdown#920](https://github.com/Python-Markdown/markdown/issues/920).
+
     On a side note: isn't `atomic_brute_cast` such a beautiful function name?
 
     Arguments:
@@ -73,62 +80,88 @@ class AutoDocProcessor(BlockProcessor):
     CLASSNAME = "autodoc"
     RE = re.compile(r"(?:^|\n)::: ?([:a-zA-Z0-9_.]*) *(?:\n|$)")
 
-    def __init__(self, parser, md: Markdown, config: dict) -> None:
+    def __init__(self, parser: BlockParser, md: Markdown, config: dict) -> None:
         """
         Initialization method.
 
         Arguments:
-            parser:
+            parser: A `markdown.blockparser.BlockParser` instance.
             md: A `markdown.Markdown` instance.
             config: The [configuration][mkdocstrings.plugin.MkdocstringsPlugin.config_scheme]
-              of the `mkdocstrings` plugin.
+                of the `mkdocstrings` plugin.
         """
         super().__init__(parser=parser)
         self.md = md
         self._config = config
 
     def test(self, parent: Element, block: Element) -> bool:
+        """Match our autodoc instructions."""
         sibling = self.lastChild(parent)
-        bool1 = self.RE.search(block)
+        bool1 = self.RE.search(str(block))
         bool2 = (
-            block.startswith(" " * self.tab_length)
+            str(block).startswith(" " * self.tab_length)
             and sibling is not None
             and sibling.get("class", "").find(self.CLASSNAME) != -1
         )
         return bool(bool1 or bool2)
 
     def run(self, parent: Element, blocks: Element) -> None:
-        block = blocks.pop(0)
-        m = self.RE.search(block)
+        """
+        The processing of autodoc instructions is done here.
 
-        if m:
-            block = block[m.end() :]  # removes the first line
+        The identifier and configuration lines are retrieved from a matched block
+        and used to collect and render an object.
+
+        Arguments:
+            parent: The parent element in the XML tree.
+            blocks: The rest of the blocks to be processed.
+        """
+        block = blocks.pop(0)
+        match = self.RE.search(str(block))
+
+        if match:
+            # removes the first line
+            block = block[match.end() :]  # type: ignore
 
         block, the_rest = self.detab(block)
 
-        if m:
-            identifier = m.group(1)
+        if match:
+            identifier = match.group(1)
             log.debug(f"mkdocstrings.extension: Matched '::: {identifier}'")
-            config = yaml.safe_load(block) or {}
+            config = yaml.safe_load(str(block)) or {}
 
             handler_name = self.get_handler_name(config)
             log.debug(f"mkdocstrings.extension: Using handler '{handler_name}'")
-            handler = get_handler(handler_name, self._config["theme_name"])
+            handler_config = self.get_handler_config(handler_name)
+            handler = get_handler(
+                handler_name,
+                self._config["theme_name"],
+                self._config["mkdocstrings"]["custom_templates"],
+                **handler_config,
+            )
 
-            selection, rendering = self.get_item_configs(handler_name, config)
+            selection, rendering = self.get_item_configs(handler_config, config)
 
             log.debug("mkdocstrings.extension: Collecting data")
             try:
-                data = handler.collector.collect(identifier, selection)
+                data: Any = handler.collector.collect(identifier, selection)
             except CollectionError:
                 log.error(f"mkdocstrings.extension: Could not collect '{identifier}'")
-                return
+                raise
 
             log.debug("mkdocstrings.extension: Updating renderer's env")
             handler.renderer.update_env(self.md, self._config)
 
             log.debug("mkdocstrings.extension: Rendering templates")
-            rendered = handler.renderer.render(data, rendering)
+            try:
+                rendered = handler.renderer.render(data, rendering)
+            except TemplateNotFound as error:
+                theme_name = self._config["theme_name"]
+                log.error(
+                    f"mkdocstrings.extension: Template '{error.name}' not found "
+                    f"for '{handler_name}' handler and theme '{theme_name}'."
+                )
+                raise
 
             log.debug("mkdocstrings.extension: Loading HTML back into XML tree")
             try:
@@ -136,9 +169,11 @@ class AutoDocProcessor(BlockProcessor):
             except ParseError as error:
                 message = f"mkdocstrings.extension: {error}"
                 if "mismatched tag" in str(error):
-                    lineno, columnno = str(error).split(":")[-1].split(", ")
-                    lineno = int(lineno.split(" ")[-1])
-                    columnno = int(columnno.split(" ")[-1])
+                    line, column = str(error).split(":")[-1].split(", ")
+
+                    lineno = int(line.split(" ")[-1])
+                    columnno = int(column.split(" ")[-1])
+
                     line = rendered.split("\n")[lineno - 1]
                     character = line[columnno]
                     message += (
@@ -147,9 +182,9 @@ class AutoDocProcessor(BlockProcessor):
                         f"or replace them with &lt; and &gt;"
                     )
                 log.error(message)
-                return
+                raise
 
-            as_xml = atomic_brute_cast(as_xml)
+            as_xml = atomic_brute_cast(as_xml)  # type: ignore
             parent.append(as_xml)
 
         if the_rest:
@@ -187,18 +222,18 @@ class AutoDocProcessor(BlockProcessor):
             return handlers.get(handler_name, {})
         return {}
 
-    def get_item_configs(self, handler_name: str, config: dict) -> Tuple[dict, dict]:
+    @staticmethod
+    def get_item_configs(handler_config: dict, config: dict) -> Tuple[dict, dict]:
         """
         Get the selection and rendering configuration merged into the global configuration of the given handler.
 
         Args:
-            handler_name: The handler to get the global configuration of.
+            handler_config: The global configuration of a handler. It can be an empty dictionary.
             config: The configuration to merge into the global handler configuration.
 
         Returns:
             Two dictionaries: selection and rendering. The local configurations are merged into the global ones.
         """
-        handler_config = self.get_handler_config(handler_name)
         item_selection_config = dict(handler_config.get("selection", {}))
         item_selection_config.update(config.get("selection", {}))
         item_rendering_config = dict(handler_config.get("rendering", {}))
@@ -219,7 +254,7 @@ class MkdocstringsExtension(Extension):
 
         Arguments:
             config: The configuration items from `mkdocs` and `mkdocstrings` that must be passed to the block processor
-              when instantiated in [`extendMarkdown`][mkdocstrings.extension.MkdocstringsExtension.extendMarkdown].
+                when instantiated in [`extendMarkdown`][mkdocstrings.extension.MkdocstringsExtension.extendMarkdown].
             kwargs: Keyword arguments used by `markdown.extensions.Extension`.
         """
         super().__init__(**kwargs)
