@@ -26,7 +26,7 @@ import logging
 import random
 import re
 import string
-from typing import Callable, List, Match, Pattern, Tuple
+from typing import Any, Callable, Dict, List, Match, Optional, Pattern, Tuple
 
 from bs4 import BeautifulSoup
 from bs4.element import NavigableString
@@ -37,10 +37,13 @@ from mkdocs.plugins import BasePlugin
 from mkdocs.structure.files import Files
 from mkdocs.structure.pages import Page
 from mkdocs.structure.toc import AnchorLink
-from mkdocs.utils import log
+from mkdocs.utils import warning_filter
 
-from .extension import MkdocstringsExtension
-from .handlers import teardown
+from mkdocstrings.extension import MkdocstringsExtension
+from mkdocstrings.handlers import teardown
+
+log = logging.getLogger(__name__)
+log.addFilter(warning_filter)
 
 SELECTION_OPTS_KEY: str = "selection"
 """The name of the selection parameter in YAML configuration blocks."""
@@ -71,9 +74,10 @@ class MkdocstringsPlugin(BasePlugin):
     """
 
     config_scheme: Tuple[Tuple[str, MkType]] = (
-        ("watch", MkType(list, default=[])),
+        ("watch", MkType(list, default=[])),  # type: ignore
         ("handlers", MkType(dict, default={})),
         ("default_handler", MkType(str, default="python")),
+        ("custom_templates", MkType(str, default=None)),
     )
     """
     The configuration options of `mkdocstrings`, written in `mkdocs.yml`.
@@ -92,9 +96,14 @@ class MkdocstringsPlugin(BasePlugin):
           handlers:
             python:
               selection:
-                selection_opt: true
+                 selection_opt: true
               rendering:
                  rendering_opt: "value"
+              setup_commands:
+                - "import os"
+                - "import django"
+                - "os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'my_djang_app.settings')"
+                - "django.setup()"
             rust:
               selection:
                 selection_opt: 2
@@ -102,23 +111,23 @@ class MkdocstringsPlugin(BasePlugin):
     """
 
     def __init__(self) -> None:
+        """Initialization method."""
         super(MkdocstringsPlugin, self).__init__()
-        self.mkdocstrings_extension = None
-        self.url_map = {}
+        self.mkdocstrings_extension: Optional[MkdocstringsExtension] = None
+        self.url_map: Dict[Any, str] = {}
 
-    def on_serve(self, server: Server, config: Config, **kwargs) -> Server:
+    def on_serve(self, server: Server, config: Config, builder: Callable = None, **kwargs) -> Server:
         """
         Hook for the [`on_serve` event](https://www.mkdocs.org/user-guide/plugins/#on_serve).
 
         In this hook, we add the directories specified in the plugin's configuration to the list of directories
         watched by `mkdocs`. Whenever a change occurs in one of these directories, the documentation is built again
         and the site reloaded.
-
-        Note:
-            The implementation is a hack. We are retrieving the watch function from a protected attribute.
-            See issue [mkdocs/mkdocs#1952](https://github.com/mkdocs/mkdocs/issues/1952) for more information.
         """
-        builder = list(server.watcher._tasks.values())[0]["func"]
+        if builder is None:
+            # The builder parameter was added in mkdocs v1.1.1.
+            # See issue https://github.com/mkdocs/mkdocs/issues/1952.
+            builder = list(server.watcher._tasks.values())[0]["func"]
         for element in self.config["watch"]:
             log.debug(f"mkdocstrings.plugin: Adding directory '{element}' to watcher")
             server.watch(element, builder)
@@ -134,14 +143,18 @@ class MkdocstringsPlugin(BasePlugin):
         We pass this plugin's configuration dictionary to the extension when instantiating it (it will need it
         later when processing markdown to get handlers and their global configurations).
         """
+        if not config["site_url"]:
+            log.error("mkdocstrings.plugin: configuration item 'site_url' is required for cross-references")
+            raise ValueError("site_url is required")
+
         log.debug("mkdocstrings.plugin: Adding extension to the list")
 
-        extension_config = dict(
-            theme_name=config["theme"].name,
-            mdx=config["markdown_extensions"],
-            mdx_configs=config["mdx_configs"],
-            mkdocstrings=self.config,
-        )
+        extension_config = {
+            "theme_name": config["theme"].name,
+            "mdx": config["markdown_extensions"],
+            "mdx_configs": config["mdx_configs"],
+            "mkdocstrings": self.config,
+        }
 
         self.mkdocstrings_extension = MkdocstringsExtension(config=extension_config)
         config["markdown_extensions"].append(self.mkdocstrings_extension)
@@ -157,7 +170,7 @@ class MkdocstringsPlugin(BasePlugin):
         """
         log.debug(f"mkdocstrings.plugin: Mapping identifiers to URLs for page {page.file.src_path}")
         for item in page.toc.items:
-            self.map_urls(page.abs_url, item)
+            self.map_urls(page.canonical_url, item)
         return html
 
     def map_urls(self, base_url: str, anchor: AnchorLink) -> None:
@@ -193,7 +206,7 @@ class MkdocstringsPlugin(BasePlugin):
         while re.search(placeholder.seed, output) or any(placeholder.seed in url for url in self.url_map.values()):
             placeholder.set_seed()
 
-        unmapped, unintended = [], []
+        unmapped, unintended = [], []  # type: ignore
         soup = BeautifulSoup(output, "html.parser")
         placeholder.replace_code_tags(soup)
         fixed_soup = AUTO_REF.sub(self.fix_ref(unmapped, unintended), str(soup))
@@ -203,8 +216,7 @@ class MkdocstringsPlugin(BasePlugin):
             if unmapped and log.isEnabledFor(logging.WARNING):
                 for ref in unmapped:
                     log.warning(
-                        f"mkdocstrings.plugin: {page.file.src_path}: Could not fix ref '[{ref}]'.\n    "
-                        f"The referenced object was not both collected and rendered."
+                        f"mkdocstrings.plugin: {page.file.src_path}: Could not find cross-reference target '[{ref}]'"
                     )
 
         return placeholder.restore_code_tags(fixed_soup)
@@ -296,8 +308,9 @@ class Placeholder:
     """
 
     def __init__(self) -> None:
-        self.ids = {}
-        self.seed = None
+        """Initialization method."""
+        self.ids: Dict[str, str] = {}
+        self.seed = ""
         self.set_seed()
 
     def store(self, value: str) -> str:
@@ -318,7 +331,7 @@ class Placeholder:
 
     def get_id(self) -> str:
         """Return a random, unique string."""
-        return f"{self.seed}{random.randint(0, 1000000)}"  # nosec: it's not for security/cryptographic purposes
+        return f"{self.seed}{random.randint(0, 1000000)}"  # noqa: S311 (it's not for security/cryptographic purposes)
 
     def set_seed(self) -> None:
         """Reset the seed in `self.seed` with a random string."""
